//          File: graph.cpp
//        Author: Elena Machkasova
// Last Modified: April 25, 2001
//         Topic: Linked List Implementation of Graphs in C++

// The file contains implementation of class Graph

#include <iostream>
#include "graph.h"

using namespace std;
// Constructor: creates graph with no vertexes
Graph::Graph() {
  first = NULL;
}
 
// Destructor: deletes the first vertex of the graph, which calls the vertex 
// destructor to delete its edges and the other vertexes with their edges 
Graph::~Graph() {
  delete first;
}

// the method returns the pointer to the vertex with data theData 
// if such vertex occurs in the graph, otherwise returns NULL
Vertex *Graph::find(char theData) {
  Vertex *vPtr = first;

  while (vPtr != NULL) {
    // check if the data of the vertex is theData
    if ( vPtr -> getData() == theData)
      return vPtr;
    // go to the next vertex
    vPtr = vPtr -> getNext();
  }

  // there has been no match in the loop, so there is no such vertex
  return NULL;
}

// the method checks if there is a vertex with data theData in the graph
// if yes, the vertex cannot be added, the method returns 'false'
// otherwise the vertex is added, the method returns 'true'
bool Graph::AddVertex(char theData) { 
  // checking if the vertex is already in the graph
  // if it is, it cannot be added again 
  if ( find(theData) != NULL )
    return false;

  // allocate memory for new vertex with data theData,
  // make it point to the previous first vertex
  Vertex *newVertex = new Vertex(theData, first);
  // make the new vertex the first one in the list of vertexes
  first = newVertex;
  return true;
}

// check if the vertexes with data Begin and End are present in the graph. 
// if yes, create an edge from Begin to End, return 'true'
// otherwise return 'false'
bool Graph::AddEdge(char Begin, char End, int w) {
  // find the pointer to the end vertex
  Vertex *vEnd = find(End);

  // if the vertex is not in the graph, cannot add the edge
  if (vEnd == NULL) return false;

  // find the pointer to the start vertex
  Vertex *vBegin = find(Begin);

  // if the vertex is not in the graph, cannot add the edge
  if (vBegin == NULL) return false;

  // connect the start vertex to the end one
  vBegin -> connectTo(vEnd, w);
  return true;
}
  
// print all vertexes and edges of the graph
void Graph::print() {
  if (first == NULL) 
    cout << "Graph has no vertexes " << endl;
  else
    first -> printGraph();
}
