//          File: vertex-edge.cpp
//        Author: Elena Machkasova
// Last Modified: April 25, 2001
//         Topic: Linked List Implementation of Graphs in C++

// The file contains implementation of classes Edge and Vertex

#include <iostream>
#include "edge.h"
#include "vertex.h"

using namespace std;
// ********************************************************************
// methods of class Vertex

// Constructor: creates a vertex with data theData that points to the next 
// vertex nextVert and has no outcoming edges
Vertex::Vertex(int theData, Vertex *nextVert) {
  data = theData;
  next = nextVert;
  edges = NULL;     // no edges 
}

// Destructor: deletes the outcoming edges (the edge destructor calls 
// destructor for the next edge) and calls destructor for the next vertex 
// of the graph 
Vertex:: ~Vertex() {
  delete next;
  delete edges;
}
  
// returns the data of the vertex
int Vertex::getData() {
  return data;
}
  
// prints all the edges of the vertex by calling the print method
// of the first edge
void Vertex::printEdges() {
  if (edges == NULL)
    cout << "vertex " << getData() << " has no edges" << endl << endl;
  else {
    cout << "vertex " << getData() << " has edges to: " << endl;
    edges -> print();
  }
}  

// prints the edges of the vertex, calls the printGraph method for 
// the next vertex
void Vertex::printGraph() {
  printEdges();
  if (next != NULL)
    next -> printGraph();
}
  
// adds an edge to connect the vertex to the vertex pointed to by Vert
void Vertex::connectTo(Vertex *Vert, int w) {
  // allocate memory for a new Edge, set its Vertex pointer to point 
  // to Vert, and its Edge pointer to point to the rest of edges 
  Edge *newEdge = new Edge(Vert, edges, w);

  // make the new edge the first edge of the vertex
  edges = newEdge; 
}

// returns the pointer to the next vertex in the graph
Vertex *Vertex::getNext() {
  return next;
}

// returns the pointer to the first outcoming edge of the vertex 
Edge *Vertex::getFirstEdge() {
  return edges;
}

// ********************************************************************
// methods of class Edge

// Constructor: sets the two fields to the two given values
Edge::Edge(Vertex *Vert, Edge *nextEdge, int w) {
  end = Vert;
  next = nextEdge;
  weight = w;
}

// Destructor: calls the destructor for the next edge on the list
Edge::~Edge() {
  delete next;
}


// returns the pointer to the end vertex of the edge
Vertex *Edge::getEnd() {
  return end;
}

int Edge::getWeight() { 
  return weight;
}

// prints out the data of the end of the edge, then calls the print method 
// of the next edge on the list (if there is one)
void Edge::print() {
  cout << end -> getData() << " with weight of " << weight << endl;
  if (next == NULL)
    cout << endl;
  else {
    next -> print();
  }
}

// returns the pointer to the next edge on the list
Edge *Edge::getNext() {
  return next;
}
